# +========================================================================+
# || CipUX:Con                                                            ||
# || The Con  library provides specific Curses and Console  System        ||
# || functions.                                                           ||
# ||                                                                      ||
# || (c) Copyright, Urheberrecht 1999 - 2005 by Christian Kuelker         ||
# || License: GPLv2                                                       ||
# || Revision: 2005-08-29                                                 ||
# +========================================================================+
package Pcis;

use 5.006000;
use strict;
use warnings;

#use Date::Calc qw(Delta_YMD Date_to_Days);
use Term::ANSIColor;
use Term::ReadKey;

use vars qw(
            $return %COLUMN %TABLE
            );

require Exporter;

our @ISA = qw(Exporter);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

# This allows declaration	use Pcis ':all';
# If you do not need this, moving things directly into @EXPORT or @EXPORT_OK
# will save memory.
our %EXPORT_TAGS = ( 'all' => [ qw(

) ] );

our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

our @EXPORT = qw(
		&search &edit &new &list
);

our $VERSION = '3.003008';

=head1 NAME

Pcis - Perl extension for Personal Console Information System

=head1 SYNOPSIS

  use Pcis;

=head1 DESCRIPTION

This module provides library functions for
the Personal Console Information System for Christian.

B<Features:>

=over

=item * search for phrases

=item * search for phrases in defined columns

=item * search for phrases in one column

=item * list all

=item * list a column

=item * edit

=item * new

=back

=head2 EXPORT

TODO


=head1 REQUIREMENTS

Configuration Files in /etc/pcis.conf

Perl modules:

DBI
DBD

dialog >=1.0
ncurses with unicode support compiled in


=head1 USAGE

Perl scripts witch would like to use Pcis.pm should have lines
like the following:

  use Pcis;

=head1 FUNCTION

The perl module Pcis.pm provides the following functions:

 * search functions
 * list functions
 * edit
 * new

=cut

# ..........................................................................
# . Global VARS
# ..........................................................................

our $SCRIPT   = "Pcis";
our $DEBUG    = 0; # DEBUG

our $red    = color("red");
our $green  = color("green");
our $blue   = color("blue");
our $black  = color("black");

# ..........................................................................
# . perform startup checks
# ..........................................................................
# chrecking HOME, and create .pcis if not there
my $home = $ENV{HOME};
my $stderr = "$home/.pcis/pcis.stderr";
if(defined($home)){
  if( -d $home){
    if( -d "$home/.pcis"){
      &log("(pcis) found home = [$home] and .pcis");
    }else{
      mkdir("$home/.pcis",0700);
      &log("(pcis) found home = [$home] and created .pcis");
    }
  }else{
    die "The HOME variable is not a directory! Specify a valid directory! Example for bash: export HOME='/home/ckuelker'\n";
  }
}else{
   die "Please define the environment variable HOME! Example for bash: export HOME='/home/ckuelker'\n";
}
# ..........................................................................
# . source in the config file
# ..........................................................................
#
# Check the config file
my $conf = "$home/.pcis/pcis.conf";
system("mkdir $home/.pcis") if ! -d "$home/.pcis";
if(-e "/etc/pcis/pcis.conf"){
  system("cp /etc/pcis/pcis.conf $conf") if ! -e $conf;
}else{
  die "Sorry, pcis was not installed correctly and therefore can not start!\n
  What may you do? \n
  (1) install pcis as root and confirm that /etc/pcis/pcis.conf is ok\n
  (2) install pcis as a normal user and confirm that ~/.pcis/pcis.conf is ok\n";
}

unless ($return = do $conf) {
    #print "RETURN [$return]\n";
    warn "="x64 . "\nERROR in '$conf': couldn't parse $conf: $@"                if $@;
    die "="x64 . "\nERROR in '$conf': couldn't do $conf: $!\n" . "="x64 . "\n" unless defined $return;
    #warn "="x64 . "\nERROR in '$conf': couldn't run $conf\n" . "="x64 . "\n"    unless $return;
}

# some dialog globals
my $dialog = " --backtitle '$SCRIPT $VERSION' --shadow";

# some generic fields (as a tail to a dialog box call)
my $fields = "";
my $l = 1;# line
foreach ( @{ $COLUMN{prs} } ){
  if($_ eq "ID" or $_ eq "date"){

  }else{
    # lastname 1 1 Lastname 1 13 25 250
    # firstname 2 1 Firstname 2 13 250 250
    $fields .= " $_ $l 1 '' $l 13 250 250";
    $l++;# line
  }
}

# ..........................................................................
# . from here on exported LIB functions
# ..........................................................................


# +========================================================================+
# ||                      Pcis->menu                                      ||
# +========================================================================+
sub create_menu{
  my $self = shift;
  my $caller = caller();
  my $cui = shift;

my $menu_file = [
    { -label => 'Quit Program',       -value => sub {exit(0)}        },
];

my $menu_pcis = [
    { -label => '    Project',        -value => sub{pcis::select_screen('Project')}  },
    { -label => 'Sub-Project',        -value => sub{select_screen(2)}  },
    { -label => '    Task',           -value => sub{select_screen(3)}  },
    { -label => '    Person',         -value => sub{pcis::select_screen('Person')}  },
    { -label => '    Telephone',      -value => sub{select_screen(5)}  },
    { -label => '    Mail',           -value => sub{select_screen(5)}  },
];

my $menu_new = [
    { -label => '    Project',        -value => sub{select_screen("Project")}  },
    { -label => 'Sub-Project',        -value => sub{select_screen(2)}  },
    { -label => '    Task',           -value => sub{select_screen(3)}  },
    { -label => '    Person',         -value => sub{select_screen(4)}  },
];


#y $demo_menu = [
#   { -label => 'Widget demos',       -submenu => $widget_demo_menu  },
#   { -label => 'Dialog demos',       -submenu => $dialog_demo_menu  },
#   { -label => '------------',       -value   => sub{}              },
#   { -label => 'Next demo',          -value   => \&goto_next_demo   },
#   { -label => 'Previous demo',      -value   => \&goto_prev_demo   },
#];

my $m = [
    { -label => 'File',               -submenu => $menu_file         },
 #   { -label => 'New',                -submenu => $menu_new          },
    { -label => 'Pcis',               -submenu => $menu_pcis         },
#   { -label => 'Edit',               -submenu => $menu_edit         },
#   { -label => 'Summarize',          -submenu => $menu_summarize    },
];

$cui->add('menu', 'Menubar', -menu => $m);
}



# +========================================================================+
# ||                      Pcis->help                                      ||
# +========================================================================+
sub create_help{
  my $self = shift;
  my $caller = caller();
  my $cui = shift;
  my $text = shift;

  my $w0 = $cui->add(
    'w0', 'Window',
    -border        => 1,
    -y             => -1,
    -height        => 3,
);
$w0->add('explain', 'Label',
  -text => "Help: $text"
);

}


sub convert_to_listbox{# global subroutine for ui
 # convert a database hash into labels (1,2,3,..)
 # and values for use with the listbox
 # USED by : person_ui, project_ui, ...

  my $db_href = shift;
  my $first   = shift;
  # $id  [ID,firstname,lastname]
  # 1   1,Christian,Kuelker
  # 2   2,Cathrin,Kuelker

  my $labels = {}; # anon hash ref
  my $values = [ ]; # anon array ref
  foreach my $id (sort keys %$db_href){
    shift(@ {  $$db_href{$id} }); # shift ID away
    # $id  [firstname,lastname]
    # 1    Christian,Kuelker
    # 2    Cathrin,Kuelker

    $$labels{$id}= join(', ',@ { $$db_href{$id} });# join one or more cols
    # "name" or "lastname, firstname"

    #$$labels{$i}= $$db_href{$i}[-1]; # last col
    #push(@$values,$i);
    &log("(convert_to_listbox): id [$id] labels_href: [$$db_href{$id}[-1]]");
  }
  foreach my $id (sort {$$labels{$a} cmp $$labels{$b}} keys %$labels){
    &log("(convert_to_listbox): labels id [$id] lables-values [$$labels{$id}]");
    if(defined($first)){# we have to consider a first value, as selected
      if($first eq $id){# put it on the first
        unshift(@$values,$id);
      }else{# put it on the last
        push(@$values,$id);
      }
    }else{# if the first is not defiened, return a alphabetacal sorted array
      push(@$values,$id);
    }
  }
  #foreach my $id (@$values){
  #   &log("(convert_to_listbox): values id [$id]");
  #}
  return $labels,$values;
}

sub add_edit{# global subroutine for ui
# add some edit fields with row header to
# the widget: generated from the database
# column headers Prj: ID name start end
# USED by : person_ui, project_ui, ...

  my $part    = shift;# prj, prs, ..
  my $s       = shift; # screen $s
  my $row     = shift; # 4 for normal # upper row
  my $column1 = shift; # 35 for prj  # left column
  my $column2 = shift; # 45 for prj  # middle column

  my $c = 0;
  foreach my $n (@ {$COLUMN{$part} }){
    &log("(add_edit): create [$n]");

    # add some left row headers
    $s->add(
      undef, 'Label',
      -y => $row+$c,
      -x => $column1,
      -bold => 1,
      -text => $n .":",
      -width => -1,
    );

    if($n eq 'ID'){
      $s->add(
        $n, 'Label',
        -y => $row+$c,
        -x => $column2,
        -bold => 0,
        -text => " - ",
        -width => -1,
      );
    }elsif($n =~ /_KEY/){
      my ($subtable,@dummy) = split('_',$n);
      &log("(add_edit): subtable [$subtable]");
      my $select = "SELECT ID,name FROM $subtable";
      &log("(add_edit): select [$select]");
      my ($labels, $values) = &convert_to_listbox(&db('select',$select));
      $s->add(
        'popupmenulabel', 'Label',
        -y => 1, -x => 2, -width => -1,
        -bold => 1,
        -text       => "none"
      );
        $s->add(
          $n, 'Listbox',
    -y          => $row+$c,
    -x          => $column2,
    -values     => $values,
    -labels     => $labels,
    -width      => 20,
    -height     => 1,
    -onselchange   => sub {
	   my $pm = shift;
	   my $lbl = $pm->parent->getobj('popupmenulabel');
	   my $val = $pm->get;
	   $val = "<undef>" unless defined $val;
	   my $lab = $pm->{-labels}->{$val};
	   $val .= " (label = '$lab')" if defined $lab;
	   $lbl->text($val);
	   $lbl->draw;
          },
  );
#      $s->add(
#        $n, 'Popupmenu',
#        -y          => $row+$c,
#        -x          => $column2,
#        -values     => $values,
#        -labels     => $labels,
#        -width      => 20,
#        -onchange   => sub {
#	   my $pm = shift;
#	   my $lbl = $pm->parent->getobj('popupmenulabel');
#	   my $val = $pm->get;
#	   $val = "<undef>" unless defined $val;
#	   my $lab = $pm->{-labels}->{$val};
#	   $val .= " (label = '$lab')" if defined $lab;
#	   $lbl->text($val);
#	   $lbl->draw;
#          },
#      );
    }else{
      $s->add(
        $n, 'TextEntry',
        -y => $row+$c,
        -x => $column2,
        -bold => 0,
        -text => "",
        -width => -1,
        -readonly => 0,
        -showlines => 1,
        -maxlength => 255,
        -sbborder => 1,
      );
    }
    $c++;
  }
}


sub update_edit{# global subroutine for ui
# get some data from the database and
# write this to the ui
# USED by : person_ui_callback, ...

  my $obj   = shift;
  my $part  = shift; # prs
  my $table = shift; # Prs
  my $key   = shift; # ID = '$key'

  my $select = "SELECT DISTINCT * FROM $table WHERE ID = '$key';";
  my $data_href = db('select',$select);
  my $c = 0;
  foreach my $i (sort keys %$data_href){
    foreach my $a (@ { $$data_href{$i} }){
      my $col = $COLUMN{$part}[$c];
      &log("(update_edit) key [$i] val [$a] c [$c] col [$col]");
      my $ref = $obj->parent->getobj($col);
      if($col =~ /_KEY/){
        #$ref->{-selected}->{$a};
        &log("(update_edit) set 1 ");
        my @l = qw (1 2 3 4);
        my $select = "SELECT DISTINCT ID,name FROM Prj";
	my ($labels, $values) = &convert_to_listbox(&db('select',$select),$a);
        $ref->labels($labels);
        $ref->values($values);

        #$lb->focus();
        $ref->set_selection([0]); # select th first one
        $ref->draw;
      }else{
        $a = '' if not defined($a);
        $ref->text($a);
      }
      $c++;
    }
  }
    #$lb->draw;
}
sub clear_edit{# global subroutine for ui
# clear the editabel field of the ui
# USED by : button_callback, ...
  my $obj = shift;
  my $part = shift;
  &log("(clear_edit): obj [$obj] part [$part]");

  foreach my $col (@ { $COLUMN{$part} }){
    &log("(clear_edit): col [$col]");
    my $ref = $obj->parent->getobj($col);
    if($col =~/_KEY/){
    }else{
      $ref->text('');
    }
  }
}



sub commit_edit{# global subroutine for ui
# get some data from the ui and write this
# to the database
# USED by : button_callback, ...

  my $obj = shift;
  my $part = shift;
  my $table = shift;
  &log("(change_edit): obj [$obj] part [$part] table [$table]");

  my @changes = ();
  my $id = 0;
  foreach my $col (@ { $COLUMN{$part} }){
    my $ref = $obj->parent->getobj($col);
        &log("(change_edit): col [$col] ref [$ref]");
    my $v =$ref->get;
    if($col eq 'ID'){
      $id = $v;
    }else{
      push(@changes,"$col = '$v'");
    }
    &log("(change_edit): col [$col] value [$v]");
  }
  my $update = "UPDATE $table SET ".join(', ',@changes)." WHERE ID ='$id';";
  &log("(project) update = [$update]");
  &db("update",$update);
}

sub insert_edit{# globals subroutine for ui
  # read every edit field (eccept ID) and
  # create in SQl insert statement from this
  my $obj = shift;
  my $part = shift;
  my $table = shift;
  &log("(commit_edit): obj [$obj] part [$part] table [$table]");

  my @changes = ();
  my $id = 0;
  my $insert = "INSERT INTO $table ("
     . join(",",@{ $COLUMN{$part} })
     . ") VALUES ( '', ";

  foreach my $col (@ { $COLUMN{$part} }){
    &log("(commit_edit): col [$col]");
    my $ref = $obj->parent->getobj($col);
    my $v = '';
    if($col ne 'ID'){
      $v =$ref->get;
      push(@changes,"'$v'");
      &log("(Pcis): col [$col] value [$v]");
    }
  }
  $insert .= join(',',@changes) . ");";
  &log("(insert_edit) insert = [$insert]");
  &db("insert",$insert);
}

sub button_callback{# global subroutine for ui
    my $obj   = shift;
    &log("(button_callback): $obj [$obj]");
    #my $part  = shift; # prs, prj, ..
    #my $table = shift; # Prs, Prj, ...
    my $value = $obj->get;
    my($action,$part,$table,$name) = split(' ', $value);
    &log("(button_callback): action [$action] part [$part]");
    &log("(button_callback): table [$table] name [$name]");
    my $button = $obj->parent->getobj($name);
    my($ui,@dummy) = split('_',$name);# person, project
    &log("(button_callback): button [$button] ui[$ui]");
    #$label->text("You pressed: " . $this->get);

    if($action eq 'change'){
      &commit_edit($obj,$part,$table);
    }
    if($action eq 'clear'){
      &clear_edit($obj,$part);
    }
    if($action eq 'insert'){
      &insert_edit($obj,$part,$table);
    }
  my $lb_name = $ui ."_ui_listbox";
  my $lb = $obj->parent->getobj($lb_name);
  &log("(button_callback): ui_listbox [$lb]");


  my $select = "SELECT DISTINCT ";
  if($table eq "Prs"){
    $select .= "ID,lastname,firstname FROM $table";
  }else{
    $select .= "ID,name FROM $table";
  }
  my ($labels, $values) = &convert_to_listbox(&db('select',$select));
  $lb->labels($labels);
  $lb->values($values);
  $lb->focus();
}

sub add_bottons{ # global subroutine for ui
  #sub add_bottons('prs','Prs',$s,'person_ui_buttons',35);
  my $part  = shift;
  my $table = shift;
  my $s     = shift;
  my $name  = shift;
  my $x     = shift;

  $s->add(
    $name, 'Buttonbox',
    -y => -1,
    -x => -1,
    -buttons => [
         {
            -label => "< change >",
        -value => "change $part $table $name",
        -onpress => \&button_callback,
        },{
            -label => "< clear >",
        -value => "clear $part $table $name",
        -onpress => \&button_callback,
        },{
            -label => "< insert >",
        -value => "insert $part $table $name",
        -onpress => \&button_callback,
        },
    ],
  );
}

# +========================================================================+
# ||                      Pcis->person_ui                                 ||
# +========================================================================+


sub person_ui_callback{ # local oubroutine for person_ui
  my $obj = shift;

  my $lb = $obj->parent->getobj('person_ui_listbox');
  &log("(person_ui_callback): person_ui_listbox [$lb]");
  my @sel = $obj->get;
  @sel = ('<none>') unless @sel;
  my $key = join("",@sel);
  &log("(person_ui_callback): key [$key]");
  &update_edit($obj,'prs','Prs',$key);
}

sub person_ui{
  my $caller = caller();
  my $self   = shift;
  my $cui    = shift;
  my $s      = shift;

  my $select = "SELECT DISTINCT ID,lastname,firstname FROM Prs";
  my ($labels, $values) = &convert_to_listbox(&db('select',$select));

  $s->add(
    'person_ui_listbox', 'Listbox',
    -y          => 2,
    -padbottom  => 2,
    -values     => $values,
    -labels     => $labels,
    -width      => 30,
    -border     => 1,
    -title      => 'Person:',
    -vscrollbar => 1,
    -onchange   => \&person_ui_callback,
  );
  &add_edit('prs',$s,4,35,45);
  &add_bottons('prs','Prs',$s,'person_ui_buttons',35);

return $s;
}


# +========================================================================+
# ||                      Pcis->project                                   ||
# +========================================================================+

sub project_ui_callback{ # local oubroutine for project_ui
  my $obj = shift;

  my $lb = $obj->parent->getobj('project_ui_listbox');
  &log("(project_ui_callback): project_ui_listbox [$lb]");
  my @sel = $obj->get;
  @sel = ('<none>') unless @sel;
  my $key = join("",@sel);
  &log("(project_ui_callback): key [$key]");
  &update_edit($obj,'prj','Prj',$key);
}

sub project_ui{
  my $caller = caller();
  my $self   = shift;
  my $cui    = shift;
  my $s      = shift;

  my $select = "SELECT DISTINCT ID,name FROM Prj";
  my ($labels, $values) = &convert_to_listbox(&db('select',$select));

  $s->add(
    'project_ui_listbox', 'Listbox',
    -y          => 2,
    -padbottom  => 2,
    -values     => $values,
    -labels     => $labels,
    -width      => 30,
    -border     => 1,
    -title      => 'Project:',
    -vscrollbar => 1,
    -onchange   => \&project_ui_callback,
  );
  &add_edit('prj',$s,4,35,45);
  &add_bottons('prj','Prj',$s,'project_ui_buttons',35);

return $s;
}

# +========================================================================+
# ||                      Pcis->init                                  ||
# +========================================================================+
sub init{
  # TODO replace with Curses::UI
  my ($h,$w) = &dialog_size();
  die "Terminal to small, need at least 24x67\n" if $h<20 or $w<64;
}

sub init_ui {
  my $caller =caller();
  my $self = shift;
  my %s = ();

  my $debug = 1;

  # Create the main window
  my $cui = new Curses::UI (
    -clear_on_exit => 1,
    -debug => $debug,
  );

my %screens = (
    '1'  => 'Project',
    '2'  => 'Person',
);

my @screens = sort {$a<=>$b} keys %screens;

my %args = (
    -border       => 1,
    -titlereverse => 0,
    -padtop       => 2,
    -padbottom    => 3,
    -ipad         => 1,
);

while (my ($nr, $title) = each %screens)
{
    &log("(Pcis): create screen nr [$nr] title [$title]");
    my $id = "window_$nr";
    $s{$title} = $cui->add(
        $id, 'Window',
        -title => "PCIS $VERSION: $title ",
        %args
    );
}
  &log("(Pcis): return cui [$cui] screen [%s]");
  return $cui,%s;
}


# +========================================================================+
# ||                      Pcis->new                                   ||
# +========================================================================+
sub new {
  my $self = shift;

  my $caller = caller();
  if($caller eq 'prs'){
    my $date = `date --iso-8601`;
    chomp($date);
    my $insert = "";
    my @input = ();
    my $exitcode = "";
    do{
      @input = &dialog('form','Input new data','Please fill out the form:',$fields);
      my $z = 1;
      $insert = "INSERT INTO Prs (".join(",",@{ $COLUMN{prs} }).") VALUES ( '', ";
      my $data = "";
      foreach (@input){
        chomp();
        &log("(new_prs) input = [$_] COLUMN = [$COLUMN{$caller}[$z]]");
        $_ = "'$_'";
        $data .= "$COLUMN{$caller}[$z]: $_\n";
        $z++;
      }
      # -yesno text height width
      $exitcode = &dialog_yesno('Check data',"Is the data ok? \n$data");
      # 0 = ok  65280 = ??
      if($exitcode == 65280){
        &log("EXITCODE = $exitcode");
        return;
      }

    }until($exitcode eq "0");# 0 = ok
    print "EXITCODE = $exitcode\n";
    $insert .= join(",",@input).", '$date');";
    &log("(new) $caller insert: [$insert]");
    &db('insert',$insert);
  }

  if($caller eq 'prj'){
    my $project = shift;
    my $enddate = shift;

    my $startdate = `date --iso-8601`;
    chomp($startdate);
    my $insert = "INSERT INTO Prj (name, start, end)";
    $insert .= "VALUES ('$project', '$startdate', '$enddate');";
    &log("(new) $caller insert: [$insert]");
    &db('insert',$insert);
    print "OK\n";
  }

  if($caller eq 'tsk'){
    my $project = shift;
    my $task    = shift;
    my $enddate = shift;
    my $minutes = shift;

    my $startdate = `date --iso-8601`;
    chomp($startdate);
    my $select = "SELECT ID FROM Prj WHERE name = '$project';";
    my $hash_ref = &db('select',$select);
    my $n = 0;
    my $key = 0;
    foreach my $id (sort keys %$hash_ref){
      &log("(new) $caller: id [$id]");
      $key = $id;
    }
    &log("(new) $caller: key = [$key]");
    if($key > 0){
      my $insert = "INSERT INTO Tsk ";
      $insert .= "(name, start, end, planned_time, used_time, Prj_KEY)";
      $insert .= "VALUES ('$task', '$startdate', '$enddate', '$minutes', '0', '$key');";
      &log("(new) $caller insert: [$insert]");
      &db('insert',$insert);
      print "OK ($key)\n";
    }else{
      print "FAILED for project '$project'\n";
    }
  }

}


# +========================================================================+
# ||                      Pcis->edit                                  ||
# +========================================================================+
sub edit{
  my $self = shift;
  my $caller  = shift;
  $caller = "prs" if not defined($caller);
  #&log("(edit) self 0 [$self] caller = [$caller]");
  my $select = "SELECT DISTINCT * FROM Prs;";
  my $hash_ref = &db('select',$select);
  my $output = "";
  my $id = 1;
  foreach (sort keys %$hash_ref){
    &log("(edit): id = ${ $$hash_ref{$_}}[0]  id = [$_] ");

    # ID 'some colums' ID 'some columns' ...
    # put a single ' in front of the second element:
    ${ $$hash_ref{$_}}[1] = " '${ $$hash_ref{$_}}[1]";
    # put a singe ' in front of the last element
    ${ $$hash_ref{$_}}[-1] .= "' ";
    $output .= join(" ", @{ $$hash_ref{$_} } );
  }
  &log("(edit): output [$output]");

  my $choice = "";
  do{
    $choice = &dialog('menu','Edit','Choose one entry to edit:',$output);
  }until($choice);

  $select = "SELECT DISTINCT * FROM Prs WHERE ID = '$choice';";
  $hash_ref = &db('select',$select);
  $output = "";
  my $r = 0;# row from array
  my $l = 1;# print line
  my $fields = "";
  my $xid = "";
  foreach my $n (sort keys %$hash_ref){
    $xid = $n;
    &log("(edit) ID to change is [$xid]");
    foreach (@ {$$hash_ref{$n}}){
      &log("(edit) COLLUMN[caller][r] = $COLUMN{$caller}[$r] = [$_]");
      if($COLUMN{$caller}[$r] ne "ID"){

        # firstname 2 1 Firstname 2 13 25 250
        $fields .= " $COLUMN{$caller}[$r] $l 1 '$_' $l 13 250 250 ";
        &log("(edit): fields = $COLUMN{$caller}[$r] $l 1 '$_' $l 13 250 250");
        $l++;
      }
      $r++;
    }
  }

  my @changes = &dialog('form','Edit',"Please change the values for ID $xid:",$fields);
  if(defined($changes[-1])){
    &log("(edit) #changes [$changes[-1]]");
    # chrate UPDATE Prs SET column='value' WHERE ID = '1';
    $r = 1;
    foreach (@changes){
      chomp();
      $_ =  "$COLUMN{$caller}[$r] = '$_'";
      $r++;
    }
    my $update = "UPDATE Prs SET ".join(', ',@changes)." WHERE ID = '$xid';";
    &log("(edit) update = [$update]");

    &db("update",$update);
  }
}

# +========================================================================+
# ||                      Pcis->mutt                                  ||
# +========================================================================+
sub mutt{
  my ($self,$search) = @_;
  my $caller = caller();
  &log("(mutt) search $search");
  # TODO: lastname+ firstname, project
  my $select = "SELECT DISTINCT email,lastname,firstname";
  $select .= " FROM Prs WHERE lastname  LIKE '%$search%'";
  $select .= " OR                firstname LIKE '%$search%'";
  $select .= " OR                email     LIKE '%$search%';";
  my $hash_ref = &db('select',$select);
  my $output = "";
  my $r = 0;
  foreach (sort keys %$hash_ref){
    $output .= join("\t", @{ $$hash_ref{$_} } ). "\n";
    $r++;
  }
  if($r > 0){
    &log("output [$output]");
    print "Search \"$search\" leads to $r hit(s).\n";
    print $output;
  }else{
    print "The string \"$search\" was not found in the database!\n";
    exit(2);
  }
}

# +========================================================================+
# ||                      Pcis->search                                ||
# +========================================================================+
sub search{
  my @o = @_;

  my $caller = caller();
  &log("(search) called by [$caller]");
  my $select = "SELECT ";
  my $cmd = "";

  &log("(search): got $#o options");
  my $self = $o[0];
  foreach(@o){
    &log("(search) got option o [$_]");
  }
  #
  # 4)
  #
  if($#o == 3){     # 3 args (+search +self)
    &log("(search): search COLUMN STRING COLUMN -> search($o[1],$o[2],$o[3])");
    $select .= "DISTINCT $o[3] FROM Prs WHERE $o[1] LIKE '%$o[2]%';";
  #
  # 3)
  #
  }elsif($#o == 2){ # 2 args (+search +self)
    &log("(search): search COLUMN STRING -> search($o[1],$o[2])");
    &log("(search): search only in column [$o[1]] after [$o[2]]");
    # SELECT DISTINCT : FROM Prs WHERE lastname like '%SEARCH%'
    $select .= "DISTINCT * FROM Prs WHERE $o[1] LIKE '%$o[2]%';";
  #
  #2)
  #
  }elsif($#o == 1){ # 1 arg (+search +self)
    &log("(search): search STRING -> search($o[1])");
    &log("(search): search in every column after [$o[1]]");
      # SELECT DISTINCT : FROM Prs WHERE lastname LIKE '%SEARCH%' OR firstname...
      my @term = @{ $COLUMN{prs} };
      foreach (@term){
        $_ = $_." LIKE '%".$o[1]."%'";
      }
      $select  .= "DISTINCT * FROM Prs WHERE ".join(' OR ',@term).";";
  #
  #1)
  #
  }else{ # 0 arg (+search +self)
    &log("(search): search -> search()");
    &log("(search): ask for every thing (search and column)");

    # DIALOG: choose columns
    # dialog $dialog --title 'Search' --checklist 'Please choose' 20 60 13 1 ID 0 2 lastname 0 3 namae 0 4 meiru 0 5 denwabango 0 6 tanjoubi 0 7 'sono hoka' 0
    my $n = 1;
    my $choose_fields = "";
    foreach (@{ $COLUMN{prs} }){
      $choose_fields .= " $n $_ 0 ";
      $n++;# number
    }
    my $choice = "";
    do{
      $choice = &dialog('checklist','Search','Please choose',$choose_fields);
    }until($choice);

    # DIALOG: enter phrase
    my $search = "";
    do {
      $search = &dialog_input('Search','Please enter a search phrase:');;
    }until($search);

    $choice =~ s/\s+$//;
    &log("=> choice [$choice]");
    my @choice = split('" "',$choice);
    my @term = ();
    foreach(@choice){
      s/"//g;
      &log("choice [$_]");
      my $temp = $_;
      my $id = $temp-1;
      &log("column [$COLUMN{$caller}[$id]]");
      push(@term, "$COLUMN{$caller}[$id] LIKE '%".$search."%'");
    }
    my $term = join(" OR ",@term);
    &log("term = [$term]");

    # SELECT DISTINCT : FROM Prs WHERE lastname LIKE '%SEARCH%' OR ...
    $select .= "DISTINCT * from Prs WHERE $term ";
  }

 # Do the selection
 my $hash_ref = &db('select',$select);
 my $output = "";
 foreach (sort keys %$hash_ref){
   $output .= join(", ", @{ $$hash_ref{$_} } ). "\n";
 }
 &log("output [$output]");

 open(FILE, ">$home/.pcis/pcis.txt") or die "Can't write to file '$home/.pcis/pcis.txt'!\n";
 print FILE $output;
 close(FILE);

 # DIALOG: Results
 &dialog_result('Results',"$home/.pcis/pcis.txt");
 &cdb($select);
}

# +========================================================================+
# ||                      Pcis->list                                      ||
# +========================================================================+
sub list{
  my $caller = caller(); # prj, prs, tsk,
  my $self   = shift;
  my $column = shift;
  my $table= $TABLE{$caller};# Prs, Prj
  &log("(list) TABLE($caller) =  [$table]");
  my $select = "SELECT DISTINCT ";
  if(defined($column)){
    $select .= " $column FROM $table;";
  }else{
    $select .= " * FROM $table;";
  }
  &cdb($select);
}
sub list_prj{
  my $select = "SELECT DISTINCT ID,name FROM Prj";
  my $hash_ref = &db('select',$select);
  return $hash_ref;
}


# +========================================================================+
# ||                      Pcis->summarize                                 ||
# +========================================================================+
sub summarize{
  my $self         = shift;
  my $project_name = shift;# NAME

  print "="x76 . "\n";
  my $caller = caller(); # prj, prs
  my $table = $TABLE{$caller};
  &log("(summarize) programme = [$caller], project = [$project_name], table = [$table]");
  my $select = "SELECT DISTINCT * from $table WHERE name = '$project_name'";
  &cdb($select);
  my $hash_ref = &db('select',$select);
  my $n = 0;
  my $key = 0;
  my %project = ();
  my %task    = ();
  foreach my $id (sort keys %$hash_ref){
    #print "Database ID: $id\n";
    $key = $id;
    foreach (@{ $$hash_ref{$id} }){
      my $format = sprintf("% 15s", $COLUMN{$caller}[$n]);
      #print "$format: $_\n";
      $project{$COLUMN{$caller}[$n]} = $_; # to save values
      $n++;
    }
  }
  if($key > 0){
    print "Assigned Tsk:\n";
    $project{planned_time} = 0;
    $project{used_time}    = 0;
    $project{end_R} = $project{end};
    $project{start_R} = "3000-01-01";
    my $planned_time = 0;
    my $used_time    = 0;
    $select = "SELECT DISTINCT * from Tsk WHERE Prj_KEY = '$key';";
    my $hash_ref2 = &db('select',$select);
    foreach my $id (sort keys %$hash_ref2){
      $n = 0;
      foreach (@{ $$hash_ref2{$id} }){
        my $colname = $COLUMN{'tsk'}[$n];
        $colname =~ s/^\s*task//g;
	$colname =~ s/^\s*name//g;
        my $format = sprintf("% 20s", "$colname $_");

        print $format if $COLUMN{'tsk'}[$n] ne 'Prj_KEY' and $COLUMN{'tsk'}[$n] ne 'ID';
        $task{$id}{$COLUMN{tsk}[$n]} = $_; # to save values
        $n++;
      }
      my $format = sprintf("% 15s", 'progress');
      my $progress = progress('percent',$task{$id}{planned_time},$task{$id}{used_time});
      if($progress > 100){
         print "$format: $red$progress%$black\n";
      }elsif($progress == 100){
         print "$format: $green$progress%$black\n";
      }else{
	 print "$format: $progress\n";
      }

      $project{planned_time} = $project{planned_time} + $task{$id}{planned_time};
      $project{used_time}    = $project{used_time}    + $task{$id}{used_time};
      $project{progress} =progress('percent',$project{planned_time},$project{used_time});
      $project{end_R}   = &max_date($project{end_R},$task{$id}{end});
      $project{start_R} = &min_date($project{start_R},$task{$id}{start});
    }
    print "-"x76 . "\n";
    #if($project{progress} > 100){
    #  print color("red"), $project{progress}
    #}else{
    #  print color("blue"), $project{progress}
    #}

    $project{progress} = 0 if not defined ($project{progress});
    $project{progress} = "$red $project{progress}% $black" if $project{progress} > 100;

    foreach my $col (sort keys %project){
      my $format = sprintf("% 15s", $col);
      if($col eq "end_R"){
        if(&end_ok($project{end},$project{end_R})){
	  print "$format: $red$project{$col}$black\n";
	}else{
	  print "$format: $green$project{$col}$black\n";
	}
      }else{
        print "$format: $project{$col}\n";
      }
    }
  }else{
    print "No project found\n";
  }

  print "="x76 . "\n";

  #&cdb($select);
  #Delta_YMD
  #           ($Dy,$Dm,$Dd) =
  #           Delta_YMD($year1,$month1,$day1,
  #                     $year2,$month2,$day2)
}

# +========================================================================+
# ||                      Pcis->min_date()                            ||
# +========================================================================+
sub progress{
  my $mode = shift; # percent
  my $max = shift;
  my $actual  = shift;
    #$actual = 1 if $actual < 1;
  &log("(progress) mode [$mode] max [$max] actual [$actual]");


  #GW=pw*100
  #GW/100 = pw
  #my $pw = $ GW/100*p%
  # prozent = 100 / ( Grundwert * prozentwert )

  #100 * .5 = 50
  #(pw/GW)*100

  my $pp = ($actual / $max) * 100;
  $pp = sprintf("%.2f", $pp);
  return  $pp;


}

# +========================================================================+
# ||                      Pcis->min_date()                            ||
# +========================================================================+
sub min_date{
  my $date1 = shift;
  my $date2 = shift;

  my ($y1,$m1,$d1) = split('-',$date1,3);
  my ($y2,$m2,$d2) = split('-',$date2,3);
  my $date = "";
  my @date = ();
  if(Date_to_Days($y1,$m1,$d1) < Date_to_Days($y2,$m2,$d2)){
    @date =  ($y1,$m1,$d1);
    &log("(max_date) $date1 < $date2");
  }else{
    @date =  ($y2,$m2,$d2);
    &log("(max_date) $date1 => $date2");
  }
  $date = sprintf("%d-%02d-%02d", @date);
  return $date;
}
# +========================================================================+
# ||                      Pcis->max_date()                            ||
# +========================================================================+
sub max_date{
  my $date1 = shift;
  my $date2 = shift;

  my ($y1,$m1,$d1) = split('-',$date1,3);
  my ($y2,$m2,$d2) = split('-',$date2,3);
  my $date = "";
  my @date = ();
  if(Date_to_Days($y1,$m1,$d1)  > Date_to_Days($y2,$m2,$d2)){
    #sprintf("%d-%02d-%02d %02d:%02d:%02d", @date)
    @date =  ($y1,$m1,$d1);
    &log("(max_date) $date1 > $date2");
  }else{
    @date =  ($y2,$m2,$d2);
    &log("(max_date) $date1 <= $date2");
  }
  $date = sprintf("%d-%02d-%02d", @date);
  return $date;
}

# +========================================================================+
# ||                      Pcis->end_ok()                       ||
# +========================================================================+
sub end_ok{
  my $date1 = shift;# project date
  my $date2 = shift;# project date R

  my ($y1,$m1,$d1) = split('-',$date1,3);
  my ($y2,$m2,$d2) = split('-',$date2,3);
  my $date = "";
  my @date = ();
  if(Date_to_Days($y1,$m1,$d1) < Date_to_Days($y2,$m2,$d2)){
    &log("(max_date) $date1 < $date2");
    return 1;# the project is more in the future as the task
  }else{
    &log("(max_date) $date1 => $date2");
    return 0;
  }
}

# +========================================================================+
# ||                      Pcis->dialog_input()                        ||
# +========================================================================+
sub dialog_input{
  my $title = shift;
  my $text = shift;

  my ($h, $w) = dialog_size();
  #my $cmd = "dialog $dialog --begin 2 2 --title '$title' --inputbox '$text' $h $w > $home/.pcis/pcis.tmp 2>&1";
  my $cmd = "dialog $dialog --begin 2 2 --title '$title' --inputbox '$text' $h $w ";
  &log("(dialog_input) [$cmd]");
  system("$cmd 2>$stderr");
  my $input = `cat $stderr`;
  chomp($input);
  return $input;
}

# +========================================================================+
# ||                      Pcis->dialog_size()                         ||
# +========================================================================+
sub dialog_size{
  my $height = "";
  my $width = "";
  my $dummy = "";
  my $cmd = "dialog --print-maxsize 3>&1 1>&2 2>&3 3>&-";
  &log("(dialog_size) cmd = [$cmd]");
  my $size = `$cmd`;
  chomp($size);
  &log("(dialog_size) [$size]");
  ($dummy, $height, $width) = split (' ', $size);
  $height =~ s/,//g;
  $height=$height-4;
  $width=$width-5;
  &log("(dialog_size) height [$height] width [$width]");
  return $height, $width;
}

# +========================================================================+
# ||                      Pcis->dialog_result()                       ||
# +========================================================================+
sub dialog_result{
  my $title = shift;
  my $file = shift;

  my ($h, $w) = dialog_size();
  #my $cmd = "dialog $dialog --begin 2 2 --title '$title' --textbox $file $h $w > $home/.pcis/pcis.tmp 2>&1";
  my $cmd = "dialog $dialog --begin 2 2 --title '$title' --textbox $file $h $w";
  &log("(dialog_result) home = [$home]\n");
  &log("(dialog_result) cmd  = [$cmd]");
  system($cmd);
}

# +========================================================================+
# ||                      Pcis->dialog()                              ||
# +========================================================================+
sub dialog{
  my $kind  = shift;# menu, checklist, form
  my $title = shift;
  my $text  = shift;
  my $fields = shift;

  my ($h, $w) = dialog_size();
  my $mh = $h-7;
  #--menu      text height width menu-height [ tag item ]
  #--checklist text height width list-height [ tag item status ]
  #--form      text height width formheight  [ label y x item y x flen ilen ]
  #--checklist 'Please choose' 20 60 13 1 ID 0 2 lastname 0 3 namae 0 4 meiru 0 5 denwabango 0

  #my $cmd = "dialog $dialog --begin 2 2 --title '$title' --$kind '$text' $h $w $mh $fields > $home/.pcis/pcis.tmp 2>&1";
  my $cmd = "dialog $dialog --single-quoted --begin 2 2 --title '$title' --$kind '$text' $h $w $mh $fields ";
  &log("(dialog) $cmd");
  system("$cmd 2>$stderr");
  if($kind eq "menu" or $kind eq "checklist"){
    my $choice = `cat $stderr`;
    chomp($choice);
    return $choice;
  }
  if($kind eq "form"){
    my @input = `cat $stderr`;
    return @input;
  }
}

# +========================================================================+
# ||                      Pcis->dialog_yesno()                        ||
# +========================================================================+
sub dialog_yesno{
  my $title = shift;
  my $text = shift;

  my ($h, $w) = dialog_size();
  # -yesno text height width
  #my $cmd = "dialog $dialog --begin 2 2 --title '$title' --yesno '$text' $h $w > $home/.pcis/pcis.tmp 2>&1";
  my $cmd = "dialog $dialog --begin 2 2 --title '$title' --yesno \"$text\" $h $w";
  &log("(dialog_yesno) $cmd");
  system($cmd);
  my $exitcode = $?;
  &log("(dialog_yesno) exitcode = [$exitcode]");
  return $exitcode;
}

# +========================================================================+
# ||                      Pcis->cdb                                   ||
# +========================================================================+
sub cdb{
  my $select = shift;

  my $p = "";
  $p = "-p '$DB_PASSWORD'" if $DB_PASSWORD ne "";
  my $cmd = "mysql -u '$DB_USER' -h '$DB_HOST' $p -e \"USE $DB; $select\"";
  &log("(cdb) cmd = [$cmd]");
  my $out = `$cmd 3>&1 1>&2 2>&3 3>&-`;
  return $out;
}

# +========================================================================+
# ||                      Pcis->db                                    ||
# +========================================================================+
sub db{
  my $mode = shift;# update, select, insert
  my $sql  = shift;

  &log("(db) mode = [$mode]");
  &log("(db) sql  = [$sql]");

  # Install the database driver
  my $drh=DBI->install_driver($DB_DRIVER) or
     die("[$SCRIPT] Couldn't load database driver '$DB_DRIVER'\n");
  &log("(db) driver installed");

  # connect to the database
  #my $dbh="";
  #  my($drh, $dsn, $username, $password, $attrhash)
  my $dbh=$drh->connect("$DB:$DB_HOST",$DB_USER,$DB_PASSWORD,{ RaiseError => 1, AutoCommit => 0 })
     or die("[$SCRIPT] Can't connect to database '$DB' ".
         "on host '$DB_HOST' with driver '$DB_DRIVER'".
         " as user '$DB_USER'\n");
  &log("(db) connected to database");

  # prepare the SELECTion
  my $cursor=$dbh->prepare($sql)
     or die("[$SCRIPT] The database preparation failed: $sql");
  &log("(db) sql prepared");

  # execute the SELECT statement
  $cursor->execute
    or die("[$SCRIPT] The SELECT statement could not be ".
           "executed: $sql\n".$cursor->errstr);
  &log("(db) sql executed");

  my %out = ();

  if($mode eq "select"){
    while( my @ary  = $cursor->fetchrow_array){
      $out{$ary[0]}=[ @ary ];
      my $t = join(' | ',@ary);
      &log("(db): row [$ary[0]], [$t]");
    }
  }

  # clean up
  $cursor->finish;
  my $result = $dbh->disconnect;
  &log("(db) database disconnected");

  if($mode eq "select"){
    return \%out;
  }
}

# +========================================================================+
# ||                      Pcis->help                                  ||
# +========================================================================+
sub help{
  my $self   = shift;
  my $caller = caller();

  print "Usage:                                | Version: $VERSION\n";
  print "                                      |\n";
  if($caller eq "prs"){
    print "       prs search                | search ...\n";
    print "       prs search PATTERN        | search for PATTERN\n";
    print "       prs search COLUMN PATTERN | serach for PATTERN in COLUMN\n";
    print "       prs list                  | lists all\n";
    print "       prs list COLUMN           | list that COLUMN\n";
    print "       prs edit                  | edit an existing dataset\n";
    print "       prs new                   | add a new dataset\n";
  #print "       prs interactive           | present a menu\n";
  }
  if($caller eq "prj"){
    print "       prj new PRJ DATE          | new project with enddate\n";
    print "       prj list                  | list all\n";
    print "       prj summarize PRJ         | project overview\n";
  }
  if($caller eq "spr"){
    print "       spr new PRJ SPR DATE      | new subproject for project with enddate\n";
    print "       spr list                  | list all\n";
    print "       spr summarize PRJ         | project overview\n";
  }
  if($caller eq "tsk"){
    print "       tsk new SPR TASK DATE TIME| new Project Task with time (minutes)\n";
    print "       tsk list                  | list all\n";
    print "       tsk summarize TASK        | task overview\n";
  }
}

# +========================================================================+
# ||                      Pcis->log                                   ||
# +========================================================================+
sub log{
  my $message = shift;
  my $caller = caller();

  chomp($message);
  my $file = "$home/.pcis/pcis.log";
  open(LOG, ">>$file") or die "Can't write to log file '$file'\n";
  my $date = `date --iso-8601`;
  chomp($date);
  print LOG "$caller $date $message\n";
  close(LOG);
}

1;

__END__

#===========================================================================
#==== Documentation ========================================================
#===========================================================================


=head1 SEE ALSO

See the CipWorX webpage and the manual at L<http://www.cipworx.org>

See the mailing list L<http://mailman.cipworx.org/listinfo/>

=head1 AUTHOR

Christian Kuelker  E<lt>christian.kuelker@cipworx.orgE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 1999-2005 by Christian Kuelker

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307 USA

=cut
