# +========================================================================+
# || CipUX::Task                                                          ||
# ||                                                                      ||
# || Defines user tasks for console and GUI                               ||
# ||                                                                      ||
# || (c) Copyright, Urheberrecht 1999 - 2005 by Christian Kuelker         ||
# ||                                                                      ||
# || License: GPLv2                                                       ||
# ||                                                                      ||
# || Revision: 2005-06-10                                                 ||
# ||                                                                      ||
# +========================================================================+
package CipUX::Task;

use 5.006000;
use strict;
use warnings;

use vars qw(@future @farfuture %cipux);

require Exporter;

our @ISA = qw(Exporter);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

# This allows declaration	use CipUX::Task ':all';
# If you do not need this, moving things directly into @EXPORT or @EXPORT_OK
# will save memory.
our %EXPORT_TAGS = ( 'all' => [ qw(

) ] );

our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
#user - course
#uid  - gid (group)

our @EXPORT = qw(
		%task
		%option
		&usage

		&cipux_task_create_object

		&cipux_task_create_user
		&cipux_task_list_users
		&cipux_task_change_user_password
		&cipux_task_backup_user_homedir
		&cipux_task_delete_user_homedir
		&cipux_task_delete_user
		&cipux_task_disable_user
		&cipux_task_enable_user
		&cipux_task_restore_user_homedir
		&cipux_task_reset_user_homedir
		&cipux_task_add_user_to_course

		&cipux_task_create_course
		&cipux_task_change_course_password
		&cipux_task_delete_course
		&cipux_task_disable_course
                &cipux_task_enable_course
		&cipux_task_list_courses
		&cipux_task_list_user_members_of_course
		&cipux_task_remove_nonexistent_users_from_all_courses
		&cipux_task_remove_user_from_course

		&cipux_task_create_role
		&cipux_task_delete_role
		&cipux_task_list_roles

		&cipux_task_create_skel
		&cipux_task_delete_skel
		&cipux_task_list_skels


                &cipux_task_list_group_node_role
		&cipux_task_list_user_node_role
		&cipux_task_list_teachers
     		&cipux_task_list_students
		&cipux_task_list_guests
		&cipux_task_list_roles
		&cipux_task_list_objects

		&cipux_task_change_cipux_config_variable_value
);
# add this feature in future
my @future = qw(

		&cipux_task_disable_course
		&cipux_task_enable_course
		&cipux_task_disable_linux_user
		&cipux_task_disable_samba_user
		&cipux_task_enable_linux_user
		&cipux_task_enable_samba_user
		&cipux_task_extent_user_exipredate
		&cipux_task_set_user_userquota
		&cipux_task_send_user_expires_warning_mail
		&cipux_task_send_user_quota_warning_mail
		&cipux_task_enable_user_accountless_printing
		&cipux_task_disable_user_acountless_printing
		&cipux_task_enable_user_accounting
		&cipux_task_enable_user_printing
		&cipux_task_disable_user_accounting
		&cipux_task_disable_user_printing
		&cipux_task_set_user_accounting_money_in_cent_for_printing
		&cipux_task_send_user_warning_mail_for_low_accounting_money
		&cipux_task_change_user_password_by_user
		&cipux_task_set_user_password_by_admin
);

# add this feature in FAR future
@farfuture =qw(
	       &cipux_task_set_user_groupquota
	       &cipux_task_add_user_to_group_as_admin
	       &cipux_task_add_user_to_group_as_jadmin
	       &cipux_task_remove_user_fom_group_as_admin
	       &cipux_task_remove_user_fom_group_jadmin
);

our $VERSION = '3.002008';

=head1 NAME

CipUX::Task - Perl extension for CipUX

=head1 SYNOPSIS

  use CipUX::Task;

=head1 DESCRIPTION

Provides the functions for cipux_task_*.

=head2 EXPORT

TODO

=cut

# +------------------------------------------------------------------------+
# | Global VARS                                                            |
# +------------------------------------------------------------------------+
my $SCRIPT = 'CipUX::Task';

my $set = '/usr/bin/cipux_set_value';
my $get = '/usr/bin/cipux_get_value';


# TODO: get this from cipux.conf
my %cipux = ();
  $cipux{'Cipux_Use_Samba'} = "no";

#  => 0,0                      : no argument to this function
#   from 1, 2, 3, ... counting
#  => 2,0: UID, CipuxFirstname   : 2 arguments = uid and firstname
#  => 2,1: UID, GID; MID         : 2 mandatory arguments, MID is auxilliary
our %task =
  (
    cipux_task_create_object           => '5,6:OID,firstname,lastname,user_node_role,group_node_role,group_type,password;default_CID,snd_skel_Uid,uhquota,usquota,ghquota,gsquota',

    cipux_task_create_user             => '5,6:UID,firstname,lastname,role,password;default_CID,snd_skel_Uid,uhquota,usquota,ghquota,gsquota',
    cipux_task_delete_user             => '1,0:UID',
    cipux_task_list_users              => '0,0:;',
    cipux_task_change_user_password    => '1,1:UID;password',
    cipux_task_backup_user_homedir     => '1,0:UID',
    cipux_task_delete_user_homedir     => '1,0:UID',
    cipux_task_disable_user            => '1,0:UID',
    cipux_task_enable_user             => '1,0:UID',
    cipux_task_restore_user_homedir    => '1,2:UID;homedirpath;skel_UID',
    cipux_task_reset_user_homedir      => '1,2:UID;homedirpath;skel_UID',
    cipux_task_add_user_to_course      => '2,0:CID,member_UID',

    cipux_task_create_course           => '2,4:CID,password;uhquota,usquota,ghquota,gsquota',
    cipux_task_delete_course           => '1,0:CID',
    cipux_task_list_courses            => '0,0:;',
    cipux_task_change_course_password  => '1,1:CID;password',
    cipux_task_disable_course          => '1,0:CID',
    cipux_task_enable_course           => '1,0:CID',

    cipux_task_list_user_members_of_course
                                       => '1,0:CID',
    cipux_task_remove_nonexistent_users_from_all_courses
                                       => '0,0:;',
    cipux_task_remove_user_from_course => '2,0:CID,member_UID',

    cipux_task_create_skel             => '1,1:SID;password',
    cipux_task_delete_skel             => '1,0:SID;',

    cipux_task_create_role             => '1,1:RID;password',
    cipux_task_delete_role             => '1,0:RID;',


     cipux_task_list_group_node_role    => '1,0:role;',
     cipux_task_list_user_node_role     => '1,0:role;',
     cipux_task_list_teachers           => '0,0:;',
     cipux_task_list_students           => '0,0:;',
     cipux_task_list_guests             => '0,0:;',
     cipux_task_list_roles              => '0,0:;',
     cipux_task_list_skels              => '0,0:;',
     cipux_task_list_objects            => '0,0:;',

  cipux_task_change_cipux_config_variable_value => '2,0:system,value;',



  );
our %option =
  (
     mode             => 'modes = cgi curses console',
     OID              => 'login: object ID, example for object = user : chkuelker',
     UID              => 'login: user ID, example: chkuelker',
     CID              => 'login: course ID, example: linuxcourse',
     SID              => 'login: skel ID, example: cycle1',
     RID              => 'login: role ID, example: student',
     password         => 'a secret',
     firstname        => 'human first name',
     lastname         => 'human family name',
     role             => 'role, like stutent, teacher, ...',
     user_node_role   => 'role like: account, share, ..',
     group_node_role  => 'role like: student, teacher, skel, ...',
     snd_skel_Uid     => 'secondary skeleton ID: example cycle1',
     default_CID      => 'default course login: course ID, group ID',
     member_UID       => 'member login of a given course: user ID, example: chkuelker',
     uhquota          => 'user hard quota',
     ghquota          => 'group hard quota',
     usquota          => 'user soft quota',
     gsquota          => 'groups soft quota',
     homedirpath      => 'path where all the homedir are, example: /skole/tjener/home0',
     skel_UID         => 'UID for the skell-user, example: student',
     system           => 'system: CAT, CipUX',
     equation         => 'some arbitrary string with a = sign, example: Cat_Feature_FAQ=1',
  );

# *************************************************************************
# * Auxilliary methodes go here                                           *
# *************************************************************************

# +------------------------------------------------------------------------+
# | split_to_first_column                                                  |
# +------------------------------------------------------------------------+
# used by
# - cipux_task_list_users
# - cipux_task_list_courses
sub split_to_first_column{
  my $out = shift;
  my @return = ();
   foreach (@$out){
    chomp();
    #$string]=~ s/\t/HALLO/g;
    my ($u1,$u2) = split("\t",$_);
    if(defined($u1)){# if it is TWO columns
      push(@return,$u1);
    }else{# it may be only ONE column
      push(@return,$_);
    }
  }
  return @return;
}

# +------------------------------------------------------------------------+
# | get_passwords_from_command_line                                        |
# +------------------------------------------------------------------------+
# used by
# - cipux_task_change_user_password
# - cipux_task_change_course_password (indirect)
sub get_password_from_command_line{
  my $password = '';
  my $password1 = '';
  my $password2 = '';

  print "please enter password: ";
  system "stty -echo";
  chomp($password1=<STDIN>);
  print "\n";
  system "stty echo";
  print "please repeat password once again: ";
  system "stty -echo";
  chomp($password2=<STDIN>);
  print "\n";
  system "stty echo";
  if($password1 eq ""){
     print "ERROR: empty password given!\n";
     exit(1);
  }
  if($password1 ne $password2){
     print "ERROR: passwords do not match!\n";
     exit(1);
  }
  $password = $password1;
  return $password;
}

sub usage{
  my $DEBUG = 0;
  #my $self = shift;
  my $caller = shift;

   print "CipWorX CipUX CiBot Task \n";
   print '-'x76 . "\n";
   print "USAGE:\n";
   my ($args_no,$args) = split(':',$task{$caller},2);
   my ($man,$aux) = split(',',$args_no,2);
   print "  $caller ";
   if(defined($args)){
     my ($mandatory,$optional) = split(';',$args,2);
     print "* mandatory [$mandatory] optional [$optional]\n" if $DEBUG;
     print "[OPTIONS]\n";
     if($man > 0){# from 0 to x, mandatory
       my(@mandatory) = split(',',$mandatory);
       #push(@mandatory,$mandatory) if defined($mandatory);
       if($#mandatory >= 0){# from -1 to x
         print "  * $man mandatory options:\n";
         foreach (@mandatory){
           print "    $_ ($option{$_})\n";
         }
       }
     }
     if($aux > 0){ # from 0 to x, auxilliary
       my(@optional)  = split(',',$optional);
       #push(@optional,$optional) if defined($optional);
       if($#optional >= 0){# from -1 to x
         print "  * $aux auxilliary options:\n";
         foreach (@optional){
           print "    $_ ($option{$_})\n";
         }
       }
     }
   }else{# if args NOT defined (nor arguments)
     print " \n";
   }
   print "  * 2 other options:\n";
   print "   -h (this page)      \n";
   print "   --help (this page)   \n";
   print "  * examples:\n";
   if($man ne 0){# we should use only man options
     my ($mandatory,$optional) = split(';',$args,2);
     $mandatory =~ s/,|;/ /g;
     print "    $caller $mandatory\n";
     if($aux ne 0){
       $args =~ s/,|;/ /g;
       print "    $caller $args\n";
     }
   }else{
     print "    $caller\n";
   }
     print "    $caller -h\n";
     print "    $caller --help\n";
     print '-'x76 . "\n";
}

# *************************************************************************
# * Preloades methodes go here                                            *
# *************************************************************************

# +------------------------------------------------------------------------+
# | cipux_task_add_member_to_course                                        |
# +------------------------------------------------------------------------+
sub cipux_task_add_user_to_course{
  # &cipux_task_add_user_to_course('cgi',$course,$member);
  my $mode       = shift;
  my $cid        = shift;
  my $member_uid = shift;

  system("$set -a -c '$cid' -e 'memberUid' -v '$member_uid'");
}

# +------------------------------------------------------------------------+
# | cipux_task_backup_user_homedir                                         |
# +------------------------------------------------------------------------+
sub cipux_task_backup_user_homedir{
  my $mode      = shift;
  my $uid       = shift;
  my $backupdir = "/skole/backup";# TODO from config file
  my $homepath  = "/skole/tjener/home0";
  my $homedir   = "/skole/tjener/home0/$uid"; # TODO from config file
  # TODO: ckeck this, and make this a config part
  my $date = `date +%F+%T`;
  chomp($date);

  mkdir $backupdir if not -d $backupdir;

  my $cmd = "cd $homepath && tar cvjf $backupdir/backup_homedir_".$uid."_".$date.".tar.bz2 $uid";
  #print "CMD [$cmd]\n";

  # create archive: UID/*
  if(-d "$homepath/$uid"){
    system($cmd);
  }
}

# +------------------------------------------------------------------------+
# | cipux_task_delete_user_homedir                                         |
# +------------------------------------------------------------------------+
sub cipux_task_delete_user_homedir{
  my $mode      = shift;
  my $uid       = shift;
  my $basedir   = "/skole/tjener/home0/"; # TODO from config file
  my $homedir   = "/skole/tjener/home0/$uid"; # TODO from config file

  if(defined($uid)){
    if($uid ne ''){
      if(length($uid) > 1){
        if($homedir ne '/'){
          if("$basedir" ne "$homedir"){
	    if(-d $homedir){
              system("rm -r $homedir");
	    }
	  }
	}
      }
    }
  }
}


# +------------------------------------------------------------------------+
# | cipux_task_change_user_password                                        |
# +------------------------------------------------------------------------+
sub cipux_task_change_user_password{
  # used by cipux_task_change_course_password
  my $mode         = shift; # cgi, curses, console
  my $uid          = shift;
  my $cml_password = shift;

  my $password = '';
  #system("/usr/bin/cipux_set_password.pl -u '$uid' -p '$password'");
  if(not defined($cml_password) and $mode eq 'console'){
    #&CipUX::CiBot::log($SCRIPT,"- Password from console");
    $password = get_password_from_command_line();
  }elsif(defined($cml_password)){
    #&log($SCRIPT,"- Password from Pramater given");
    $password = $cml_password;
  }else{
    #&log($SCRIPT,"- WARNING: got not defined password, using empty password");
  }
  if($cipux{Cipux_Use_Samba} eq "yes"){
    #&log($SCRIPT,"- Cipux_Use_Samba = yes");
    # This also will add the ldap values
    open(CMD,"| smbpasswd -s $uid >>/var/log/cipux/cipux::task.pm.log 2>&1");
    print CMD "$password\n$password\n";
    close(CMD);
  }else{
    #&log($SCRIPT,"- Cipux_Use_Samba = no");
  }
  my $modsalt = join '', ('.','/',0..9,'A'..'Z','a'..'z')[rand 64,rand 64];
  my $crypt_password = crypt($password,$modsalt);

  system("$set -u '$uid' -e 'userPassword' -v '{crypt}$crypt_password'");
  system("$set -c '$uid' -e 'userPassword' -v '{crypt}$crypt_password'");

  #&log($SCRIPT,"- Set password for [$uid]");
}

# +------------------------------------------------------------------------+
# | cipux_task_change_course_password                                        |
# +------------------------------------------------------------------------+
sub cipux_task_change_course_password{
  my $mode         = shift; # cgi, curses, console
  my $cid          = shift;
  my $cml_password = shift;

  &cipux_task_change_user_password($mode,$cid,$cml_password);
}

# +------------------------------------------------------------------------+
# | cipux_task_reset_user_homedir                                          |
# +------------------------------------------------------------------------+
sub cipux_task_reset_user_homedir{
  # 2 = reset = delete and create
  my $mode         = shift;
  # man
  my $uid          = shift;
  # opt
  my $homedirpath  = shift;
  my $skeluser     = shift;

  $homedirpath = '/skole/tjener/home0' if not defined($homedirpath);
  if(not defined($skeluser)){
  # TODO: check the SYSTEM
    $skeluser = `$get -c $uid -e cipuxRole`;
    chomp($skeluser);
  }

  #  2    /skole/tjener/home0 teachers wbolte
  # MODE HOMEDIRPATH         SKELUSER USER
  `echo "cipux_task_reset_user_homedir: /usr/bin/cipux_refine_homedir reset '$homedirpath' '$skeluser' '$uid'" >> /var/log/cipux/CipUX::Task.log`;
  system("/usr/bin/cipux_refine_homedir reset '$homedirpath' '$skeluser' '$uid'");

  if($skeluser eq 'student'){
    my $forbachskel = `$get -u $uid -e cipuxSkeletonUid`;
    chomp($forbachskel);
    `echo "cipux_task_reset_user_homedir: role is stutent: using forbachskel [$forbachskel]" >> /var/log/cipux/CipUX::Task.log`;
    # WARNING: we must use restore, or onle the last refine will remain
     system("/usr/bin/cipux_refine_homedir restore '$homedirpath' '$forbachskel' '$uid'");

  }

}

# +------------------------------------------------------------------------+
# | cipux_task_restore_user_homedir                                          |
# +------------------------------------------------------------------------+
sub cipux_task_restore_user_homedir{
  # 1 = restore = rsync only the config data
  my $mode     = shift;
  # man
  my $uid      = shift;
  # opt
  my $homedirpath  = shift;
  my $skeluser = shift;

  $homedirpath = '/skole/tjener/home0' if not defined($homedirpath);
  if(not defined($skeluser)){
  # TODO: check the SYSTEM
    $skeluser = `$get -c $uid -e cipuxRole`;
    chomp($skeluser);
  }

  #  1    /skole/tjener/home0 teachers wbolte
  # MODE  HOMEDIRPATH         SKELUSER USER
  `echo "cipux_task_reset_user_homedir: /usr/bin/cipux_refine_homedir restore '$homedirpath' '$skeluser' '$uid'" >> /var/log/cipux/CipUX::Task.log`;
  system("/usr/bin/cipux_refine_homedir restore '$homedirpath' '$skeluser' '$uid'");

  if($skeluser eq 'student'){
    my $forbachskel = `$get -u $uid -e cipuxSkeletonUid`;
    chomp($forbachskel);
    `echo "cipux_task_restore_user_homedir: role is stutent: using forbachskel [$forbachskel]" >> /var/log/cipux/CipUX::Task.log`;
     system("/usr/bin/cipux_refine_homedir restore '$homedirpath' '$forbachskel' '$uid'");

  }
}

# +------------------------------------------------------------------------+
# | cipux_task_create_course                                               |
# +------------------------------------------------------------------------+
sub cipux_task_create_course{
 # &cipux_task_create_course('cgi',$password,$course,$hquota,$squota,$type);
  my $mode       = shift; # cgi
  # man:
  my $id         = shift; # grammar   group: cn, gid, course_login; user: uid, login
  my $password   = shift; # secret
  # opt 1:
  my $uhquota    = shift; # 1000
  my $usquota    = shift; # 800
  # opt 2:
  my $gsquota    = shift; # 1800
  my $ghquota    = shift; # 2000
  #  default
  my $role1       = 'share';
  my $role2       = 'course';
  my $group_type  = 'public';
  my $homedirpath = '/skole/tjener/home0';
  # aux:
  #my $group_type = shift; # school_class   private
  #my $role       = shift; # course         teacher
  # my $firstname  = shift; # cipux
  # my $lastname   = shift; # course
  # my $role       = shift; # course
  # my $group      = shift;
  # my $course     = shift;

  # create group
  my $c_attr = " --attribute cn='$id'";
  $c_attr   .= " --attribute groupType='$group_type' ";
  $c_attr   .= " --attribute cipuxSoftQuota='$gsquota'" if defined($gsquota);
  $c_attr   .= " --attribute cipuxHardQuota='$ghquota'" if defined($ghquota);
  $c_attr   .= " --attribute cipuxRole='$role2'";
  $c_attr   .= " --attribute cipuxInternetStatus=accept";
  $c_attr   .= " --attribute cipuxRemark='created by CipUX::Task ($mode)'";

  system("/usr/bin/cipux_add -c -w '$password' $c_attr");

  # create login
  my $u_attr = " --attribute uid='$id'";
  $u_attr   .= " --attribute cipuxSoftQuota='$usquota'" if defined($usquota);
  $u_attr   .= " --attribute cipuxHardQuota='$uhquota'" if defined($uhquota);
  $u_attr   .= " --attribute cipuxRole='$role1'";
  $u_attr   .= " --attribute cipuxInternetStatus=accept";
  $u_attr   .= " --attribute cipuxRemark='created by CipUX::Task ($mode)'";
  $u_attr   .= " --attribute cipuxFirstname='CipUX'";
  $u_attr   .= " --attribute cipuxLastname='course'";

  system("/usr/bin/cipux_add -u -w '$password' $u_attr");

  # make only an refinement to skel if this is one of the main roles


  #  2    /skole/tjener/home0 teachers wbolte
  #                                   MODE HOMEDIRPATH   SKELUSER USER
  system("/usr/bin/cipux_refine_homedir reset '$homedirpath' '$role2' '$id'");

}

# +------------------------------------------------------------------------+
# | cipux_task_create_examination_course                                   |
# +------------------------------------------------------------------------+
sub cipux_task_create_examination_course{
  my $mode       = shift; # cgi
  my $password   = shift; # secret
  my $id         = shift; # grammar   group: cn, gid, course_login; user: uid, login
  my $uhquota    = shift; # 1000
  my $usquota    = shift; # 800
  my $group_type = shift; # school_exam
  my $role       = shift; # examination
  my $leader     = shift; # bibeutlin
  my $course     = shift;

  # TODO attr !!!

  system("/usr/bin/cipux_add -c -w '$password' --attribute cn='$course' --attribute groupType='school_exam' --attribute cipuxSoftQuota='$usquota' --attribute cipuxHardQuota='$uhquota' --attribute cipuxGroupLeaderUid='$leader' --attribute cipuxRole='examination' --attribute cipuxInternetStatus=accept");
  # create login
  system("/usr/bin/cipux_add -u -w '$password' --attribute cipuxHardQuota='$uhquota' --attribute cipuxSoftQuota='$usquota)' --attribute cipuxFirstname='cipux' --attribute cipuxLastname='examination' --attribute cipuxRole='course' --attribute cipuxInternetStatus=accept --attribute uid='$course' ");

}

# +------------------------------------------------------------------------+
# | cipux_task_create_skel                                                 |
# +------------------------------------------------------------------------+
sub cipux_task_create_skel{
  my $mode = shift;
  # mandatory
  my $login = shift;
  # optional
  my $password =shift;

  &cipux_task_create_user('cgi',
    $login,
    'CipUX',
    "Skel UID",
    'skel',
    $password);

  # TODO make a create_object subroutine
  my $cmd = "cipux_set_value -c $login -e groupType -v shared";
  #print "cipux_set_value -c $login -e groupType -v shared";
  `$cmd`;
  # TODO make password optional


}
# +------------------------------------------------------------------------+
# | cipux_task_create_role                                                 |
# +------------------------------------------------------------------------+
sub cipux_task_create_role{
  my $mode = shift;
  # mandatory
  my $login = shift;
  # optional
  my $password =shift;

  &cipux_task_create_user('cgi',
    $login,
    'CipUX',
    "Role",
    'none',
    $password);

  # TODO make a create_object subroutine
  my $cmd = "cipux_set_value -c $login -e groupType -v shared";
  `$cmd`;
  $cmd = "cipux_set_value -c $login -e cipuxRole -v skel";
  `$cmd`;
  $cmd = "cipux_set_value -u $login -e cipuxRole -v role";
  `$cmd`;

  # TODO make password optional


}

# +------------------------------------------------------------------------+
# | cipux_task_create_user                                                 |
# +------------------------------------------------------------------------+
sub cipux_task_create_user{
  # &cipux_task_create_user('cgi',$password,$login,$hquota,$squota,$role,$firstname,$lastname);
  my $mode         = shift; # cgi
  # man:
  my $id           = shift; # grammar   group: cn, gid, course_login; user: uid, login
  my $firstname    = shift; # cipux
  my $lastname     = shift; # course
  my $role         = shift; # student
  my $password     = shift; # secret
  # opt 1:
  my $default_CID  = shift; # chemie   memberUid # default group
  my $snd_skel_Uid = shift; # secondary skeleton ID: cycle1
  my $uhquota      = shift; # 1000
  my $usquota      = shift; # 800
  # opt 2:
  my $ghquota      = shift; # 2000
  my $gsquota      = shift; # 1800
  #my $group_type  = shift; # private
  #my $group       = shift;# chemie   memberUid # default group
  my $group_type   = 'private';
  my $dn           = "ou=People,dc=skole,dc=skolelinux,dc=no";


  my $args = join(' ',@_);
  system("echo 'cipux_task_create_user: $id,$firstname,$lastname,$role' >> /var/log/cipux/CipUX::Task.log");
  system("echo ' ' >> /var/log/cipux/CipUX::Task.log");


  if($role eq "student"
    or $role eq "teacher"
    or $role eq "guest"
    or $role eq "tutor"
    or $role eq "examenee"
    or $role eq "assistent"
    or $role eq "skel"
    or $role eq "admin"
    or $role eq "none"){

    # add course
    my $c_attr = "  --attribute cn='$id'";
    $c_attr   .= " --attribute groupType='$group_type' ";
    $c_attr   .= " --attribute cipuxSoftQuota='$gsquota'" if defined($gsquota);
    $c_attr   .= " --attribute cipuxHardQuota='$ghquota'" if defined($ghquota);
    $c_attr   .= " --attribute cipuxRole='$role'";
    $c_attr   .= " --attribute cipuxInternetStatus=accept";
    $c_attr   .= " --attribute memberUid='$id'";
    # $c_attr   .= " --attribute member='uid=$id,$dn'"; # QUESTION: not more in sage debian-edu?
    $c_attr   .= " --attribute cipuxRemark='created by CipUX::Task ($mode)'";

    system("/usr/bin/cipux_add -c -w '$password' $c_attr");

    # add user
    my $u_attr = " --attribute uid='$id'";
    $u_attr   .= " --attribute cipuxSoftQuota='$usquota'" if defined($usquota);
    $u_attr   .= " --attribute cipuxHardQuota='$uhquota'" if defined($uhquota);
    $u_attr   .= " --attribute cipuxSkeletonUid='$snd_skel_Uid'" if defined($snd_skel_Uid);
    $u_attr   .= " --attribute cipuxRole='account'"; # policy 2005-08-05
    $u_attr   .= " --attribute cipuxInternetStatus=accept";
    $u_attr   .= " --attribute cipuxRemark='created by CipUX::Task ($mode)'";
    $u_attr   .= " --attribute cipuxFirstname='$firstname'";
    $u_attr   .= " --attribute cipuxLastname='$lastname'";

    system("/usr/bin/cipux_add -u -w '$password' $u_attr");

    # normal skeleton
    if($role ne "none"){
      my $cmd = "/usr/bin/cipux_refine_homedir restore '/skole/tjener/home0' $role $id";
      system("echo '* normal skeleton: $cmd' >> /var/log/cipux/CipUX::Task.log");
      system($cmd);
    }

    # Forbach skeleton
    if(defined($snd_skel_Uid)){
      my $cmd = "/usr/bin/cipux_refine_homedir restore '/skole/tjener/home0' $snd_skel_Uid $id";
      system("echo '* forbach skeleton: $cmd' >> /var/log/cipux/CipUX::Task.log");
      system($cmd);
    }

    # add to user to role group
    system("$set -a -c '$role' -e memberUid -v '$id'");
    #system("$set -a -c '$role' -e member -v 'uid=$id,$dn'");#QUESTION: not more in sage debian-edu?

    # add user to default class
    if(defined($default_CID)){
      # add to default group (for class, or exam)
      system("$set -a -c '$default_CID' -e memberUid -v '$id'");
      #system("$set -a -c '$default_CID' -e member -v 'uid=$id,$dn'");#QUESTION: not more in sage debian-edu?
    }

  }else{
    print "ERROR: can't create role $role!\n" if $mode eq 'console' ;
  }

  # '$password' '$login' '$firstname '$lastname' '$role' '$uhquota' '$usquota' '$ghquota' 'gsquota' '$group'
  #system("/usr/bin/cipux_add -c -w '$password' --attribute cn='$id' --attribute groupType='private' --attribute cipuxSoftQuota='$usquota' --attribute cipuxHardQuota='$uhquota' --attribute cipuxGroupLeaderUid='$leader' --attribute cipuxRole='examination' --attribute cipuxInternetStatus=accept");
  #system("/usr/bin/cipux_add -c -w '$password' --attribute cn='$id' --attribute groupType='private' --attribute cipuxSoftQuota='$usquota' --attribute cipuxHardQuota='$uhquota' --attribute --attribute cipuxRole='student' --attribute cipuxInternetStatus=accept");

  # create login
  #system("/usr/bin/cipux_add -u -w '$password' --attribute cipuxHardQuota='$uhquota' --attribute cipuxSoftQuota='$usquota)' --attribute cipuxFirstname='cipux' --attribute cipuxLastname='examination' --attribute cipuxRole='course' --attribute cipuxInternetStatus=accept --attribute uid='$id' ");

  # add user to role group (ROLE+s)
  #echo "$set -a -c $role.'s' -e memberUid -v '$id'";
  #system("$set -a -c $role -e memberUid -v '$id'");

  # add user to primary group? TODO: neccessary?
  #echo "$set -a -c '$id' --e memberUid -v '$id'";
  #$set -a -c '$id' --e memberUid -v '$id'
  #system("$set -a -c 'id' --e memberUid -v '$id'");

  # add user to default group TODO: if its a student
  #echo "$set -a -c '$group' --e memberUid -v '$id'";
  #$set -a -c '$group' --e memberUid -v '$id'

}

# +------------------------------------------------------------------------+
# | cipux_task_create_object                                                 |
# +------------------------------------------------------------------------+
sub cipux_task_create_object{
  my $mode         = shift; # cgi
  # man:
  my $id           = shift; # grammar   group: cn, gid, course_login; user: uid, login
  my $firstname    = shift; # cipux     cipux     cipux
  my $lastname     = shift; # user      role      course
  my $urole        = shift; # account | role    | share
  my $grole        = shift; # student | skel    |
  my $gtype        = shift; # private | shared  | public
  my $password     = shift; # secret
  # opt 1:
  my $default_CID  = shift; # chemie   memberUid # default group
  my $snd_skel_Uid = shift; # secondary skeleton ID: cycle1
  my $uhquota      = shift; # 1000
  my $usquota      = shift; # 800
  # opt 2:
  my $ghquota      = shift; # 2000
  my $gsquota      = shift; # 1800
  my $dn           = "ou=People,dc=skole,dc=skolelinux,dc=no";

  system("echo 'cipux_task_create_object($id,$firstname,$lastname,$urole,$grole,$gtype)' >> /var/log/cipux/CipUX::Task.log");
  system("echo ' ' >> /var/log/cipux/CipUX::Task.log");


  if($grole eq "student"
    or $grole eq "teacher"
    or $grole eq "guest"
    or $grole eq "tutor"
    or $grole eq "examenee"
    or $grole eq "assistent"
    or $grole eq "skel"
    or $grole eq "admin"
    or $grole eq "none"){

    # add course
    my $c_attr = "  --attribute cn='$id'";
    $c_attr   .= " --attribute groupType='$gtype' ";
    $c_attr   .= " --attribute cipuxSoftQuota='$gsquota'" if defined($gsquota);
    $c_attr   .= " --attribute cipuxHardQuota='$ghquota'" if defined($ghquota);
    $c_attr   .= " --attribute cipuxRole='$grole'";
    $c_attr   .= " --attribute cipuxInternetStatus=accept";
    $c_attr   .= " --attribute memberUid='$id'";
    # $c_attr   .= " --attribute member='uid=$id,$dn'"; # QUESTION: not more in sage debian-edu?
    $c_attr   .= " --attribute cipuxRemark='created by CipUX::Task ($mode)'";

    #print "/usr/bin/cipux_add -c -w '$password' $c_attr\n";
    system("/usr/bin/cipux_add -c -w '$password' $c_attr");

    # add user
    my $u_attr = " --attribute uid='$id'";
    $u_attr   .= " --attribute cipuxSoftQuota='$usquota'" if defined($usquota);
    $u_attr   .= " --attribute cipuxHardQuota='$uhquota'" if defined($uhquota);
    $u_attr   .= " --attribute cipuxSkeletonUid='$snd_skel_Uid'" if defined($snd_skel_Uid);
    $u_attr   .= " --attribute cipuxRole='$urole'";
    $u_attr   .= " --attribute cipuxInternetStatus=accept";
    $u_attr   .= " --attribute cipuxRemark='created by CipUX::Task ($mode)'";
    $u_attr   .= " --attribute cipuxFirstname='$firstname'";
    $u_attr   .= " --attribute cipuxLastname='$lastname'";

    system("/usr/bin/cipux_add -u -w '$password' $u_attr");

    # normal skeleton
    if($grole ne "none"){
      my $cmd = "/usr/bin/cipux_refine_homedir restore '/skole/tjener/home0' $grole $id";
      system("echo '* normal skeleton: $cmd' >> /var/log/cipux/CipUX::Task.log");
      system($cmd);
    }

    # Forbach skeleton
    if(defined($snd_skel_Uid)){
      my $cmd = "/usr/bin/cipux_refine_homedir restore '/skole/tjener/home0' $snd_skel_Uid $id";
      system("echo '* forbach skeleton: $cmd' >> /var/log/cipux/CipUX::Task.log");
      system($cmd);
    }

    # add to user to role group
    system("$set -a -c '$grole' -e memberUid -v '$id'");
    #system("$set -a -c '$role' -e member -v 'uid=$id,$dn'");#QUESTION: not more in sage debian-edu?

    # add user to default class
    if(defined($default_CID)){
      # add to default group (for class, or exam)
      system("$set -a -c '$default_CID' -e memberUid -v '$id'");
      #system("$set -a -c '$default_CID' -e member -v 'uid=$id,$dn'");#QUESTION: not more in sage debian-edu?
    }

  }else{
    print "ERROR: can't create role $grole!\n" if $mode eq 'console' ;
  }

}



# +------------------------------------------------------------------------+
# | cipux_task_create_examination_user                                                 |
# +------------------------------------------------------------------------+
sub cipux_task_create_examination_user{
  # &cipux_task_create_examination_user('cgi',$password,$login,$hquota,$squota,$role,$firstname,$lastname);
  my $mode       = shift; # cgi
  my $password   = shift; # secret
  my $id         = shift; # grammar   group: cn, gid, course_login; user: uid, login
  my $uhquota    = shift; # 1000
  my $usquota    = shift; # 800
  # my $ghquota    = shift; # 2000
  # my $gsquota    = shift; # 1800
  #  my $group_type = shift; # private
  my $role       = shift; # student
  my $firstname  = shift; # cipux
  my $lastname   = shift; # course
  my $group      = shift;# chemie   memberUid
  my $leader     = shift;
  my $course     = shift;

  # '$password' '$login' '$firstname '$lastname' '$role' '$uhquota' '$usquota' '$ghquota' 'gsquota' '$group'
  system("/usr/bin/cipux_add -c -w '$password' --attribute cn='$course' --attribute groupType='private' --attribute cipuxSoftQuota='$usquota' --attribute cipuxHardQuota='$uhquota' --attribute cipuxGroupLeaderUid='$leader' --attribute cipuxRole='examination' --attribute cipuxInternetStatus=accept");

  # create login
  system("/usr/bin/cipux_add -u -w '$password' --attribute cipuxHardQuota='$uhquota' --attribute cipuxSoftQuota='$usquota)' --attribute cipuxFirstname='cipux' --attribute cipuxLastname='examination' --attribute cipuxRole='course' --attribute cipuxInternetStatus=accept --attribute uid='$course' ");

}

# +------------------------------------------------------------------------+
# | cipux_task_delete_course                                               |
# +------------------------------------------------------------------------+
sub cipux_task_delete_course{
  my $mode   = shift;
  my $cid    = shift;
  &cipux_task_delete_user($mode,$cid);
  print "DELETE";
}

# +------------------------------------------------------------------------+
# | cipux_task_delete_skel                                                 |
# +------------------------------------------------------------------------+
sub cipux_task_delete_skel{
  my $mode   = shift;
  my $cid    = shift;
  &cipux_task_delete_user($mode,$cid);
  print "DELETE";
}

# +------------------------------------------------------------------------+
# | cipux_task_delete_role                                                 |
# +------------------------------------------------------------------------+
sub cipux_task_delete_role{
  my $mode   = shift;
  my $cid    = shift;
  &cipux_task_delete_user($mode,$cid);
  print "DELETE";
}

# +------------------------------------------------------------------------+
# | cipux_task_delete_user                                                 |
# +------------------------------------------------------------------------+
sub cipux_task_delete_user{
  my $mode = shift;
  my $uid  = shift;
  &cipux_task_backup_user_homedir($mode,$uid);
  &cipux_task_delete_user_homedir($mode,$uid);
  system("/usr/bin/cipux_del -c '$uid'");
  system("/usr/bin/cipux_del -u '$uid'");
}

# +------------------------------------------------------------------------+
# | cipux_task_disable_user                                                |
# +------------------------------------------------------------------------+
sub cipux_task_disable_user{
  my $mode = shift;
  my $uid  = shift;
  my $attr = 'userPassword';
  my $u_pwd = `cipux_get_value -u '$uid' -e $attr`;
  my $c_pwd = `cipux_get_value -c '$uid' -e $attr`;
  chomp($u_pwd);
  chomp($c_pwd);
  system("$set -u $uid -e $attr -v '[DISABLED]$u_pwd'");
  system("$set -c $uid -e $attr -v '[DISABLED]$c_pwd'");
}

# +------------------------------------------------------------------------+
# | cipux_task_disable_course                                               |
# +------------------------------------------------------------------------+
sub cipux_task_disable_course{
  my $mode = shift;
  my $cid  = shift;
  &cipux_task_disable_user($mode,$cid);
}

# +------------------------------------------------------------------------+
# | cipux_task_enable_user                                                 |
# +------------------------------------------------------------------------+
sub cipux_task_enable_user{
  my $mode = shift;
  my $uid  = shift;
  my $attr = 'userPassword';
  my $u_pwd = `cipux_get_value -u '$uid' -e $attr`;
  my $c_pwd = `cipux_get_value -c '$uid' -e $attr`;
  chomp($u_pwd);
  chomp($c_pwd);
  $u_pwd =~ s/^\[DISABLED\]//;
  $c_pwd =~ s/^\[DISABLED\]//;
  system("$set -u $uid -e $attr -v '$u_pwd'");
  system("$set -c $uid -e $attr -v '$c_pwd'");
}

# +------------------------------------------------------------------------+
# | cipux_task_enable_course                                               |
# +------------------------------------------------------------------------+
sub cipux_task_enable_course{
  my $mode = shift;
  my $cid  = shift;
  &cipux_task_enable_user($mode,$cid);
}

# +------------------------------------------------------------------------+
# | cipux_task_list_courses                                                |
# +------------------------------------------------------------------------+
sub cipux_task_list_courses{
  my $mode  = shift;

  my @role = `$get -U -e cipuxRole`;
  my @return =();
  foreach (@role){
    chomp;
    my($u,$r) = split(/\t/);
    push(@return,$u) if $r eq 'share';
  }

  return @return;
}


# +------------------------------------------------------------------------+
# | cipux_task_list_user_node_role                                                   |
# +------------------------------------------------------------------------+
sub cipux_task_list_user_node_role{
  my $mode  = shift;
  # mandatory
  my $role  = shift;# share, account

  my @role = `$get -U -e cipuxRole`;
  my @return =();
  foreach (@role){
    chomp;
    my($u,$r) = split(/\t/);
    push(@return,$u) if $r eq $role;
  }
  return @return;
}

# +------------------------------------------------------------------------+
# | cipux_task_list_roles                                                            |
# +------------------------------------------------------------------------+

sub cipux_task_list_roles{
  my $mode  = shift;
  return cipux_task_list_user_node_role('task','role');

}

# +------------------------------------------------------------------------+
# | cipux_task_list_skels                                                            |
# +------------------------------------------------------------------------+

sub cipux_task_list_skels{
  my $mode  = shift;
  return cipux_task_list_group_node_role('task','skel');
}


# +------------------------------------------------------------------------+
# | cipux_task_list_group_node_role                                                   |
# +------------------------------------------------------------------------+
sub cipux_task_list_group_node_role{
  my $mode  = shift;
  # mandatory
  my $role  = shift;# student, teacher

  my @role = `$get -C -e cipuxRole`;
  my @return =();
  foreach (@role){
    chomp;
    my($u,$r) = split(/\t/);
    push(@return,$u) if $r eq $role;
  }
  return @return;
}


# +------------------------------------------------------------------------+
# | cipux_task_list_teachers                                               |
# +------------------------------------------------------------------------+
sub cipux_task_list_teachers{
  my $mode  = shift;
  cipux_task_list_group_node_role('task','teacher');
}

# +------------------------------------------------------------------------+
# | cipux_task_list_students                                               |
# +------------------------------------------------------------------------+
sub cipux_task_list_students{
  my $mode  = shift;
  cipux_task_list_group_node_role('task','student');
}

# +------------------------------------------------------------------------+
# | cipux_task_list_guest                                                  |
# +------------------------------------------------------------------------+
sub cipux_task_list_guests{
  my $mode  = shift;
  cipux_task_list_group_node_role('task','guest');
}


# +------------------------------------------------------------------------+
# |   cipux_task_list_user_members_of_course                               |
# +------------------------------------------------------------------------+
sub cipux_task_list_user_members_of_course{
  my $mode = shift;
  my $cid  = shift;

  my @out = `$get -c '$cid' -e memberUid`;
  my @list = &split_to_first_column(\@out);
  my @return = ();
  foreach(@list){

    # only REAL member are interesting
    push(@return,$_) if $_ ne $cid;
  }
  return @return;
}

# +------------------------------------------------------------------------+
# | cipux_task_list_objects                                                  |
# +------------------------------------------------------------------------+
sub cipux_task_list_objects{
  my $mode  = shift; # cgi

  #my @out    = `$get -U -e 'uid'`;
  my @uid  = `$get -U -e uid`;
  my @return =();
  foreach (@uid){
    chomp;
    my($u,$r) = split(/\t/);
    push(@return,$u) if $u ne 'root';
  }

  #my @return = &split_to_first_column(\@out);
  return @return;
}

# +------------------------------------------------------------------------+
# | cipux_task_list_users                                                  |
# +------------------------------------------------------------------------+
sub cipux_task_list_users{
  my $mode  = shift; # cgi

  #my @out    = `$get -U -e 'uid'`;
  my @role  = `$get -C -e groupType`;
  my @return =();
  foreach (@role){
    chomp;
    my($u,$r) = split(/\t/);
    push(@return,$u) if $r eq 'private';
  }

  #my @return = &split_to_first_column(\@out);
  return @return;
}

# +------------------------------------------------------------------------+
# | cipux_task_remove_nonexistent_users_from_all_courses                   |
# +------------------------------------------------------------------------+
sub cipux_task_remove_nonexistent_users_from_all_courses{
  my $mode = shift;

  system("/usr/bin/cipux_bot_extinct_from_courses -a");
}

# +------------------------------------------------------------------------+
# | cipux_task_remove_user_from_course                                     |
# +------------------------------------------------------------------------+
sub cipux_task_remove_user_from_course{
  my $mode       = shift;
  my $cid        = shift;
  my $member_uid = shift;

  system("$set -d -c '$cid' -e 'memberUid' -v '$member_uid'");
}

# +------------------------------------------------------------------------+
# | cipux_task_remove_user_from_all_courses                                     |
# +------------------------------------------------------------------------+
sub cipux_task_remove_user_from_all_courses{
  my $mode = shift;
  my $uid  = shift;
  system("/usr/bin/cipux_extinct_from_courses -u '$uid'");
}


# +------------------------------------------------------------------------+
# | cipux_task_enable_internet_for_course                                  |
# +------------------------------------------------------------------------+
sub cipux_task_enable_internet_for_course{
  my $mode   = shift;
  my $course  = shift;
  system("/usr/bin/cipux_internet_access -e -c '$course'");
}

# +------------------------------------------------------------------------+
# | cipux_task_disable_internet_for_course                                  |
# +------------------------------------------------------------------------+
sub cipux_task_disable_internet_for_course{
  my $mode   = shift;
  my $course  = shift;
  system("/usr/bin/cipux_internet_access -d -c '$course'");
}

# +------------------------------------------------------------------------+
# | cipux_task_start_exam                                  |
# +------------------------------------------------------------------------+
sub cipux_task_start_exam{
  my $mode    = shift;
  my $course  = shift;
  my $teacher = shift;

  system("/usr/bin/cipux_start_exam $course $teacher ");
}

# +------------------------------------------------------------------------+
# | cipux_task_collect_exam                                  |
# +------------------------------------------------------------------------+
sub cipux_task_collect_exam{
  my $mode    = shift;
  my $course  = shift;
  my $teacher = shift;
  system("/usr/bin/cipux_collect_exam $course $teacher ");
}

# +------------------------------------------------------------------------+
# | cipux_task_end_exam                                  |
# +------------------------------------------------------------------------+
sub cipux_task_end_exam{
  my $mode    = shift;
  my $course  = shift;
  my $teacher = shift;
  system("/usr/bin/cipux_end_exam $course $teacher ");
}

# +------------------------------------------------------------------------+
# | cipux_task_change_cipux_config_variable_value                          |
# +------------------------------------------------------------------------+
sub cipux_task_change_cipux_config_variable_value {
  my $mode    = shift;
  my $system  = shift; # cipux, cibot
  my $value   = shift; # $in{cipux_variable}=$in{cipux_variable_value}
  system("$set -a -x '$system' -e cipuxVariable -v '$value'");
}

1;

__END__

#===========================================================================
#==== Documentation ========================================================
#===========================================================================

=head1 SEE ALSO

See the CipUX webpage and the  manual at L<http://www.cipux.org>
See the mailing list L<http://www.cipux.org/mailman/listinfo/>

=head1 AUTHOR

Christian Kuelker  E<lt>christian.kuelker@cipworx.orgE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 1999-2005 by Christian Kuelker

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307 USA

=cut
